@@ -7,11 +7,11 @@ import hashlib
 import re
 import os
 import fnmatch
-import time
 import email.utils
 import email.policy
 import requests
 import urllib.parse
+import datetime
 
 from tempfile import mkstemp
 
@@ -19,7 +19,7 @@ from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-VERSION = '0.3.2'
+VERSION = '0.3.3'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -75,9 +75,11 @@ DEFAULT_CONFIG = {
     # strict: must match one of the uids on the key to pass
     # loose: any valid and trusted key will be accepted
     'attestation-uid-match': 'loose',
+    # How many days before we consider attestation too old?
+    'attestation-staleness-days': '30',
     # NB! This whole behaviour will change once public-inbox
     # gains support for cross-list searches
-    'attestation-query-url': LOREADDR + '/signatures',
+    'attestation-query-url': LOREADDR + '/signatures/',
     # We'll use the default gnupg homedir, unless you set it here
     'attestation-gnupghome': None,
     # Do you like simple or fancy checkmarks?
@@ -323,7 +325,7 @@ class LoreSeries:
         if lmsg is None:
             return 'undefined'
 
-        prefix = time.strftime('%Y%m%d', lmsg.date[:9])
+        prefix = lmsg.date.strftime('%Y%m%d')
         authorline = email.utils.getaddresses(lmsg.msg.get_all('from', []))[0]
         author = re.sub(r'\W+', '_', authorline[1]).strip('_').lower()
         slug = '%s_%s' % (prefix, author)
@@ -345,6 +347,10 @@ class LoreSeries:
 
         attdata = [None] * self.expected
         attpolicy = config['attestation-policy']
+        try:
+            attstaled = int(config['attestation-staleness-days'])
+        except ValueError:
+            attstaled = 30
         exact_from_match = False
         if config['attestation-uid-match'] == 'strict':
             exact_from_match = True
@@ -357,6 +363,7 @@ class LoreSeries:
             attfail = FAIL_SIMPLE
 
         at = 1
+        atterrors = list()
         for lmsg in self.patches[1:]:
             if lmsg is not None:
                 if self.has_cover and covertrailers and self.patches[0].followup_trailers:
@@ -375,11 +382,35 @@ class LoreSeries:
                     if attdoc is None:
                         if attpolicy in ('softfail', 'hardfail'):
                             logger.info('  %s %s', attfail, lmsg.full_subject)
+                            # Which part failed?
+                            failed = ['commit metadata', 'commit message', 'patch content']
+                            for attdoc in ATTESTATIONS:
+                                for i, m, p in attdoc.hashes:
+                                    if p == lmsg.attestation.p:
+                                        failed.remove('patch content')
+                                    if m == lmsg.attestation.m:
+                                        failed.remove('commit message')
+                                    if i == lmsg.attestation.i:
+                                        failed.remove('commit metadata')
+                            atterrors.append('Patch %s/%s failed attestation (%s)' % (at, lmsg.expected,
+                                                                                      ', '.join(failed)))
                         else:
                             logger.info('  %s', lmsg.full_subject)
                     else:
-                        logger.info('  %s %s', attpass, lmsg.full_subject)
-                        attdata[at-1] = attdoc.attestor.get_trailer(lmsg.fromemail)
+                        if attpolicy == 'check':
+                            # switch to softfail policy now that we have at least one hit
+                            attpolicy = 'softfail'
+                        # Make sure it's not too old compared to the message date
+                        # Timezone doesn't matter as we calculate whole days
+                        tdelta = lmsg.date.replace(tzinfo=None) - attdoc.sigdate
+                        if tdelta.days > attstaled:
+                            # Uh-oh, attestation is too old!
+                            logger.info('  %s %s', attfail, lmsg.full_subject)
+                            atterrors.append('Attestation for %s/%s is over %sd old: %sd' % (at, lmsg.expected,
+                                                                                             attstaled, tdelta.days))
+                        else:
+                            logger.info('  %s %s', attpass, lmsg.full_subject)
+                            attdata[at-1] = attdoc.attestor.get_trailer(lmsg.fromemail)
                 else:
                     logger.info('  %s', lmsg.full_subject)
 
@@ -402,7 +433,7 @@ class LoreSeries:
                 logger.info('  %s %s', attpass, trailer)
             return mbx
 
-        errors = set()
+        errors = set(atterrors)
         for attdoc in ATTESTATIONS:
             errors.update(attdoc.errors)
 
@@ -410,7 +441,7 @@ class LoreSeries:
             logger.critical('  ---')
             logger.critical('  Attestation is available, but did not succeed:')
             for error in errors:
-                logger.critical('  %s %s', attfail, error)
+                logger.critical('    %s %s', attfail, error)
 
         if attpolicy == 'hardfail':
             import sys
@@ -481,7 +512,7 @@ class LoreMessage:
         except IndexError:
             pass
 
-        self.date = email.utils.parsedate_tz(str(self.msg['Date']))
+        self.date = email.utils.parsedate_to_datetime(str(self.msg['Date']))
 
         diffre = re.compile(r'^(---.*\n\+\+\+|GIT binary patch)', re.M | re.I)
         diffstatre = re.compile(r'^\s*\d+ file.*\d+ (insertion|deletion)', re.M | re.I)
@@ -935,6 +966,7 @@ class LoreAttestationDocument:
         self.good = False
         self.valid = False
         self.trusted = False
+        self.sigdate = None
         self.passing = False
         self.attestor = None
         self.hashes = set()
@@ -956,16 +988,20 @@ class LoreAttestationDocument:
                 keyid = gs_matches.groups()[0]
                 self.attestor = LoreAttestor(keyid)
                 puid = '%s <%s>' % self.attestor.get_primary_uid()
-                if re.search(r'^\[GNUPG:\] VALIDSIG', output, re.M):
+                vs_matches = re.search(r'^\[GNUPG:\] VALIDSIG ([0-9A-F]+) (\d{4}-\d{2}-\d{2}) (\d+)', output, re.M)
+                if vs_matches:
                     logger.debug('  VALIDSIG')
                     self.valid = True
+                    ymd = vs_matches.groups()[1]
+                    self.sigdate = datetime.datetime.strptime(ymd, '%Y-%m-%d')
                     # Do we have a TRUST_(FULLY|ULTIMATE)?
-                    matches = re.search(r'^\[GNUPG:\] TRUST_(FULLY|ULTIMATE)', output, re.M)
-                    if matches:
-                        logger.debug('  TRUST_%s', matches.groups()[0])
+                    ts_matches = re.search(r'^\[GNUPG:\] TRUST_(FULLY|ULTIMATE)', output, re.M)
+                    if ts_matches:
+                        logger.debug('  TRUST_%s', ts_matches.groups()[0])
                         self.trusted = True
                     else:
-                        self.errors.add('Insufficient trust on key: %s (%s)' % (keyid, puid))
+                        self.errors.add('Insufficient trust (model=%s): %s (%s)'
+                                        % (config['attestation-trust-model'], keyid, puid))
                 else:
                     self.errors.add('Signature not valid from key: %s (%s)' % (keyid, puid))
         else:
@@ -1070,15 +1106,13 @@ class LoreAttestation:
                 self.passing = True
                 self.attdocs.append(attdoc)
 
-        if len(self.attdocs) or not lore_lookup:
-            return
-
-        attdocs = LoreAttestationDocument.get_from_lore(self.attid)
-        ATTESTATIONS += attdocs
-        for attdoc in attdocs:
-            if hg in attdoc.hashes and attdoc.passing:
-                self.passing = True
-                self.attdocs.append(attdoc)
+        if not len(self.attdocs) and lore_lookup:
+            attdocs = LoreAttestationDocument.get_from_lore(self.attid)
+            ATTESTATIONS += attdocs
+            for attdoc in attdocs:
+                if hg in attdoc.hashes and attdoc.passing:
+                    self.passing = True
+                    self.attdocs.append(attdoc)
 
     def __repr__(self):
         out = list()
